@@ -7,9 +7,6 @@
 extern "C" {
 #endif
 
-/* This is about the type 'xrange', not the built-in function range(), which
-   returns regular lists. */
-
 /*
 A range object represents an integer range.  This is an immutable object;
 a range cannot change its value after creation.
@@ -19,10 +16,10 @@ they are represented by a start, stop, and step datamembers.
 */
 
 PyAPI_DATA(PyTypeObject) PyRange_Type;
+PyAPI_DATA(PyTypeObject) PyRangeIter_Type;
+PyAPI_DATA(PyTypeObject) PyLongRangeIter_Type;
 
-#define PyRange_Check(op) ((op)->ob_type == &PyRange_Type)
-
-PyAPI_FUNC(PyObject *) PyRange_New(long, long, long, int);
+#define PyRange_Check(op) (Py_TYPE(op) == &PyRange_Type)
 
 #ifdef __cplusplus
 }
